###
### Script to run to obtain Figure 2
###

source("function_univ_os_int.R")
source("functions_sample_mean_sd.R")

Q <- 1:100;
N <- 4*Q+1;

distribution <- "Normal"

if(distribution == "Normal"){
  mu.norm <- 50; sig.norm <- 17;
  mean <- mu.norm; sd <- sig.norm;
}
if(distribution == "l-Normal"){
  mu.lnorm <- 4 ; sig.lnorm <- 0.3;
  mean <- exp(mu.lnorm + sig.lnorm^2/2) ; sd <- sqrt( (exp(sig.lnorm^2)-1) * exp( 2*mu.lnorm + sig.lnorm^2) );  
}

parastart.norm <- c(mean, sd)
parastart.lnorm <- c(log(mean)-log(sd^2/mean^2+1)/2, sqrt(log(sd^2/mean^2+1)))

nrep <- 10000;
set.seed(14342)

data.mean.est <- symb5norm.mean.est <- symb5lnorm.mean.est <- sample.mean.est <-  matrix( nrow=nrep, ncol=length(Q) )
data.sd.est <- symb5norm.sd.est <- symb5lnorm.sd.est <- sample.sd.est1 <- sample.sd.est2 <- matrix( nrow=nrep, ncol=length(Q) )

pb = txtProgressBar(min = 0, max = length(Q), initial = 0, style=3) #creates a progress bar

for(i in 1:length(Q)){
  n <- N[i];
  #cat("\n n=",n,"\n")
  
  setTxtProgressBar(pb, i)
  
  for(j in 1:nrep){
    
    if(distribution == "Normal"){
      tmp <- sort(rnorm(n=n, mean=mu.norm, sd=sig.norm))  
    }
    if(distribution == "l-Normal"){
      tmp <- sort(rlnorm(n=n, meanlog=mu.lnorm, sd=sig.lnorm))
    }
    
    data.mean.est[j,i] <- mean(tmp)
    data.sd.est[j,i] <- sd(tmp)    
  
    # 5 Number symbol:
    Symb.5 <- list(n=n)
    Symb.5$type <- "5Num"
    #Symb.5$coord <- quantile(tmp, probs=prob3)
    Symb.5$orders <- c(1, 1 + 1:3*(n-1)/4, n) # Q = (n-1)/4
    Symb.5$coord <- tmp[Symb.5$orders]
    
    # fit5.norm <- fit.univ.os.int(symbol.data = list(Symb.5), parastart=parastart.norm, hessian=FALSE, trace=0)
    fit5.norm <- fit.univ.os.int(symbol.data = list(Symb.5), distribution="Normal", parastart=parastart.norm, hessian=FALSE, trace=0)
    symb5norm.mean.est[j,i] <- fit5.norm$est[1]
    symb5norm.sd.est[j,i] <- fit5.norm$est[2] * sqrt(n/(n-1))
    
    if(distribution=="l-Normal"){
      # fit5.lnorm <- fit.univ.os.int.lnorm(symbol.data = list(Symb.5), parastart=parastart.lnorm, hessian=FALSE, trace=0)
      fit5.lnorm <- fit.univ.os.int(symbol.data = list(Symb.5), distribution="l-Normal", parastart=parastart.lnorm, hessian=FALSE, trace=0)
      symb5lnorm.mean.est[j,i] <- exp(fit5.lnorm$est[1] + fit5.lnorm$est[2]^2/2)
      symb5lnorm.sd.est[j,i] <- sqrt( (exp(fit5.lnorm$est[2]^2)-1) * exp( 2*fit5.lnorm$est[1] + fit5.lnorm$est[2]^2) ) * sqrt(n/(n-1))
    }
         
    sample.mean.est[j,i] <- est.sample.mean(quantiles=Symb.5$coord, n=n)
    sample.sd.est1[j,i] <- est.sample.sd(quantiles=Symb.5$coord, n=n, method="Wan")
    sample.sd.est2[j,i] <- est.sample.sd(quantiles=Symb.5$coord, n=n, method="Shi")    
    
  }
  
}

filename=paste("Estimation_",distribution,"_5num_Mu",mu.norm,"_Sig",sig.norm,"_Nrep",nrep,"_Sample_Mean_Sd_estimates.RData",sep="")
#save(symb5norm.mean.est, symb5norm.sd.est, symb5lnorm.mean.est, symb5lnorm.sd.est, symb5snorm.mean.est, symb5snorm.sd.est, sample.mean.est, sample.sd.est1, sample.sd.est2, data.mean.est, data.sd.est, mean, sd, file=filename)
if(distribution=="l-Normal"){
  save(symb5norm.mean.est, symb5norm.sd.est, symb5lnorm.mean.est, symb5lnorm.sd.est, sample.mean.est, sample.sd.est1, sample.sd.est2, data.mean.est, data.sd.est, mean, sd, file=filename)
}
if(distribution=="Normal"){
  save(symb5norm.mean.est, symb5norm.sd.est, sample.mean.est, sample.sd.est1, sample.sd.est2, data.mean.est, data.sd.est, mean, sd, file=filename)
}

###
###
### Illustrations (Figure 2)
###
###

load(filename)

diff.mean.sample <- apply(sample.mean.est-data.mean.est,2,mean)
diff.mean.sample.se <- apply(sample.mean.est-data.mean.est,2,sd) / sqrt(nrep)
diff.sd.sample1 <- apply(sample.sd.est1-data.sd.est,2,mean)
diff.sd.sample2 <- apply(sample.sd.est2-data.sd.est,2,mean)
diff.sd.sample1.se <- apply(sample.sd.est1-data.sd.est,2,sd) / sqrt(nrep)
diff.sd.sample2.se <- apply(sample.sd.est2-data.sd.est,2,sd) / sqrt(nrep)

diff.mean.symb5norm <- apply(symb5norm.mean.est-data.mean.est,2,mean)
diff.mean.symb5norm.se <- apply(symb5norm.mean.est-data.mean.est,2,sd) / sqrt(nrep)
diff.sd.symb5norm <- apply(symb5norm.sd.est-data.sd.est,2,mean)
diff.sd.symb5norm.se <- apply(symb5norm.sd.est-data.sd.est,2,sd) / sqrt(nrep)

if(distribution=="l-Normal"){
  diff.mean.symb5lnorm <- apply(symb5lnorm.mean.est-data.mean.est,2,mean)
  diff.mean.symb5lnorm.se <- apply(symb5lnorm.mean.est-data.mean.est,2,sd) / sqrt(nrep)  
  diff.sd.symb5lnorm <- apply(symb5lnorm.sd.est-data.sd.est,2,mean)
  diff.sd.symb5lnorm.se <- apply(symb5lnorm.sd.est-data.sd.est,2,sd) / sqrt(nrep)
}


if(distribution=="Normal"){
  main.x <- expression(paste("Normal data, ",bar(x)))
  main.s <- expression(paste("Normal data, ",s))
  ylim1 <- range(c(diff.mean.symb5norm - 2*diff.mean.symb5norm.se, diff.mean.sample - 2*diff.mean.sample.se, diff.mean.symb5norm + 2*diff.mean.symb5norm.se, diff.mean.sample + 2*diff.mean.sample.se),finite=TRUE)
  ylim2 <- range(c(diff.sd.symb5norm - 2*diff.sd.symb5norm.se, diff.sd.sample1 - 2*diff.sd.sample1.se, diff.sd.sample2 - 2*diff.sd.sample2.se, diff.sd.symb5norm + 2*diff.sd.symb5norm.se, diff.sd.sample1 + 2*diff.sd.sample1.se, diff.sd.sample2 + 2*diff.sd.sample2.se),finite=TRUE)
  xlim <- c(1,200)
}
if(distribution=="l-Normal"){
  main.x <- expression(paste("LogNormal data, ",bar(x)))
  main.s <- expression(paste("LogNormal data, ",s))
  ylim1 <- range(c(diff.mean.symb5norm - 2*diff.mean.symb5norm.se, diff.mean.symb5lnorm - 2*diff.mean.symb5lnorm.se, diff.mean.sample - 2*diff.mean.sample.se, diff.mean.symb5norm + 2*diff.mean.symb5norm.se, diff.mean.symb5lnorm + 2*diff.mean.symb5lnorm.se, diff.mean.sample + 2*diff.mean.sample.se),finite=TRUE)
  ylim2 <- range(c(diff.sd.symb5norm - 2*diff.sd.symb5norm.se, diff.sd.symb5lnorm - 2*diff.sd.symb5lnorm.se, diff.sd.sample1 - 2*diff.sd.sample1.se, diff.sd.sample2 - 2*diff.sd.sample2.se, diff.sd.symb5norm + 2*diff.sd.symb5norm.se, diff.sd.symb5lnorm + 2*diff.sd.symb5lnorm.se, diff.sd.sample1 + 2*diff.sd.sample1.se, diff.sd.sample2 + 2*diff.sd.sample2.se),finite=TRUE)
  xlim <- c(1,400)
}


plot(N, diff.mean.symb5norm, xlim=xlim, ylim=ylim1, type="b", pch=16, col="green", main=main.x, ylab="Mean difference error", xlab="Sample size, n",cex.main = 1.65, cex.lab = 1.65, cex.axis = 1.4 )
arrows(N, diff.mean.symb5norm - 2*diff.mean.symb5norm.se, N, diff.mean.symb5norm + 2*diff.mean.symb5norm.se, length=0.025, angle=90, code=3, col="green")
lines(N,diff.mean.sample, col="red", pch=15, type="b", lty=2, cex=0.75)
arrows(N, diff.mean.sample - 2*diff.mean.sample.se, N, diff.mean.sample + 2*diff.mean.sample.se, length=0.025, angle=90, code=3, col="red")
if(distribution=="l-Normal"){
  lines(N,diff.mean.symb5lnorm, col="darkgreen", pch=16, type="b", lty=2, cex=0.75)
  arrows(N, diff.mean.symb5lnorm - 2*diff.mean.symb5lnorm.se, N, diff.mean.symb5lnorm + 2*diff.mean.symb5lnorm.se, length=0.025, angle=90, code=3, col="darkgreen")
}
abline(h=0, lty=3, lwd=2)

plot(N, diff.sd.symb5norm, xlim=xlim, ylim=ylim2, type="b", pch=16, col="green", main=main.s, ylab="Mean difference error", xlab="Sample size, n",cex.main = 1.65, cex.lab = 1.65, cex.axis = 1.4 )
arrows(N, diff.sd.symb5norm - 2*diff.sd.symb5norm.se, N, diff.sd.symb5norm + 2*diff.sd.symb5norm.se, length=0.025, angle=90, code=3, col="green")
lines(N,diff.sd.sample1, col="blue", pch=17, type="b", lty=2, cex=0.75)
arrows(N, diff.sd.sample1 - 2*diff.sd.sample1.se, N, diff.sd.sample1 + 2*diff.sd.sample1.se, length=0.025, angle=90, code=3, col="blue")
lines(N,diff.sd.sample2, col="purple", pch=17, type="b", lty=2, cex=0.75)
arrows(N, diff.sd.sample2 - 2*diff.sd.sample2.se, N, diff.sd.sample2 + 2*diff.sd.sample2.se, length=0.025, angle=90, code=3, col="purple")
if(distribution=="l-Normal"){
  lines(N,diff.sd.symb5lnorm, col="darkgreen", pch=16, type="b", lty=2, cex=0.75)
  arrows(N, diff.sd.symb5lnorm - 2*diff.sd.symb5lnorm.se, N, diff.sd.symb5lnorm + 2*diff.sd.symb5lnorm.se, length=0.025, angle=90, code=3, col="darkgreen")
}
abline(h=0, lty=3, lwd=2)



