###
### Script to run to obtain Figure 3
###

source("function_univ_os_int.R")
# source("functions_sample_mean_sd.R")

Q <- c(5, 20, 50);
N <- 4*Q+1;

distribution <- "Normal"
mean <- 50; sd <- 17;
parastart.norm <- c(mean, sd)

nrep <- 10000;
set.seed(14342)


for(k in 1:length(Q)){
  
  cat("Sample size:", Q[k], "\n")
  n <- N[k];
  pb = txtProgressBar(min = 0, max = 2*Q[k], initial = 0, style=3) #creates a progress bar
  
  data.mean.est <- symb3norm.mean.est <- symb2norm.mean.est <- matrix( nrow=nrep, ncol=2*Q[k] )
  data.sd.est <- symb3norm.sd.est <- symb2norm.sd.est <- matrix( nrow=nrep, ncol=2*Q[k] )
  
  for(i in 1:(2*Q[k])){
    
    setTxtProgressBar(pb, i)
    
    for(j in 1:nrep){
      
      tmp <- sort(rnorm(n=n, mean=mean, sd=sd))  
      
      data.mean.est[j,i] <- mean(tmp)
      data.sd.est[j,i] <- sd(tmp)    
      
      # 3 Number symbol:
      Symb.3 <- list(orders = c( i, 2*Q[k]+1, n+1-i ))
      Symb.3$coord <- tmp[Symb.3$orders]
      Symb.3$n <- n

      # 2 Number symbol:
      Symb.2 <- list(orders = c(i, n+1-i))
      Symb.2$coord <- tmp[Symb.2$orders]
      Symb.2$n <- n

      fit3.norm <- fit.univ.os.int(symbol.data = list(Symb.3), distribution="Normal", parastart=parastart.norm, hessian=FALSE, trace=0)
      symb3norm.mean.est[j,i] <- fit3.norm$est[1]
      symb3norm.sd.est[j,i] <- fit3.norm$est[2] * sqrt(n/(n-1))

      fit2.norm <- fit.univ.os.int(symbol.data = list(Symb.2), distribution="Normal", parastart=parastart.norm, hessian=FALSE, trace=0)
      symb2norm.mean.est[j,i] <- fit2.norm$est[1]
      symb2norm.sd.est[j,i] <- fit2.norm$est[2] * sqrt(n/(n-1))

    }
    
  }
  
  save(data.mean.est, data.sd.est, symb3norm.mean.est, symb2norm.mean.est, symb3norm.sd.est, symb2norm.sd.est, file=paste("fig3_n",n,".RData", sep="") )
  
}


###
###
### Illustrations
###
###

LTY <- c(1, 5, 3)

for(k in 1:length(Q)){
  
  n <- N[k];
  load(file=paste("fig3_n",n,".RData", sep=""))
  
  # Calculate RMSE
  RMSE.3.mean <- apply( (symb3norm.mean.est-mean)^2, 2, sum) / apply( (data.mean.est-mean)^2, 2, sum)
  RMSE.2.mean <- apply( (symb2norm.mean.est-mean)^2, 2, sum) / apply( (data.mean.est-mean)^2, 2, sum)
  
  q <- (n+1 - 1:(2*Q[k]))/n
  if(k ==1){
    plot( rev(q), rev(RMSE.3.mean), type="l", lty=LTY[k], ylim=c(0.95,3), xlim=c(0.5,1), xlab="Quantile", ylab="RMSE", lwd=2)
  }else{
    lines(rev(q), rev(RMSE.3.mean), lty=LTY[k], lwd=2)
  }
  
  lines(rev(q), rev(RMSE.2.mean), lty=LTY[k], col="grey", lwd=2)

}  


for(k in 1:length(Q)){
  
  n <- N[k];
  load(file=paste("fig3_n",n,".RData", sep=""))
  
  # Calculate RMSE
  RMSE.3.sd <- apply( (symb3norm.sd.est-sd)^2, 2, sum) / apply( (data.sd.est-sd)^2, 2, sum)
  RMSE.2.sd <- apply( (symb2norm.sd.est-sd)^2, 2, sum) / apply( (data.sd.est-sd)^2, 2, sum)
  
  q <- (n+1 - 1:(2*Q[k]))/n
  if(k ==1){
    plot( rev(q), rev(RMSE.3.sd), type="l", lty=LTY[k], ylim=c(0.95,8), xlim=c(0.5,1), xlab="Quantile", ylab="RMSE", lwd=2)
  }else{
    lines(rev(q), rev(RMSE.3.sd), lty=LTY[k], lwd=2)
  }
  
  lines(rev(q), rev(RMSE.2.sd), lty=LTY[k], col="grey",lwd=2)
  
}  




