library(MASS)
library(mvtnorm)

rm(list=ls())
source("functions_biv_int.R") # Make sure to be in the right directory!
source("functions_biv_int_full.R")


m <- c(20, 50) # number of symbols
ns <- c(5, 10, 100, 1000, 100000) # Number of points per symbols
rho <- c(0, 0.3, 0.5, 0.7, 0.9) # Correlations

params <- expand.grid(ns,rho,m)

mean <- c(2,5)
var <- c(0.5^2, 0.5^2)

nrep <- 100 # number of replicates for each parameter setting

# IF RUNNING ON SERVER
args <- commandArgs(TRUE)
job.id <- as.numeric(args[1])-16 # Should take value 1,...,4000
param.selected <- ceiling(job.id/nrep)
replicate <- job.id - (param.selected-1) * nrep

# IF NOT RUNNING ON SERVER THEN SET param.selected and replicate
# param.selected <- 1
# replicate <- 1

# Parameters consider by the job

param.vector <- as.matrix(params[param.selected,])
ns <- param.vector[1]
Cor <- param.vector[2]
m <- param.vector[3]

Cov <- Cor * prod(sqrt(var)) # Covariance
sigma <- matrix(c(var[1], Cov, Cov, var[2]), ncol=2)

# Creating the symbols

symbol.data.bare <- vector("list", length = m)

set.seed(100)

j <- 1
while( j<=replicate ){
  
  n2 <- n3 <- n4 <- vector(length=m)
  for(i in 1:m){
  	
    tmp <- mvrnorm(ns, mean, Sigma = sigma )    
    coord <- c(range(tmp[,1]), range(tmp[,2])) # a1, a2, b1, b2    
    symbol.data.bare[[i]] <- list(coord=coord, n=ns)
  }  
  j <- j +1
}

Time.full <- system.time(
  fit.full <- fit.biv.int.full(symbol.data = symbol.data.bare, parastart = c(mean, var[1], Cov,var[2]), hessian=FALSE, trace=2)
)

est.full <- c(fit.full$est, fit.full$est[4]/sqrt(prod(fit.full$est[c(3,5)])), Time.full[3])

save(est.full, file= paste("N_",m, "Bare_Symb_", ns, "PtsSymb_",Cor,"Cor_","rep_", replicate,".RData",sep=""))


