
library(MASS)
library(mvtnorm)

rm(list=ls())
source("functions_biv_int_cor.R") # Make sure to be in the right directory!

m <- c(20, 50) # number of symbols
ns <- c(5, 10, 100, 1000, 100000)
rho <- c(0, 0.3, 0.5, 0.7, 0.9) # Correlation

params <- expand.grid(ns,rho,m)

mean <- c(2,5)
sd <- c(0.5, 0.5)
var <- sd^2

nrep <- 100 # number of replicates for each parameter setting

# IF RUNNING ON SERVER
args <- commandArgs(TRUE) # A value from 1 to 40, giving the parameter scenario
# IF NOT RUNNING ON SERVER THEN SET args
# args <- 1

# Parameters consider by the job
param.vector <- as.matrix(params[args,])

ns <- param.vector[1]
Cor <- param.vector[2]
m <- param.vector[3]

Cov <- Cor * prod(sd)
sigma <- matrix(c(var[1], Cov, Cov, var[2]), ncol=2)

est <- est.4 <- matrix(ncol=6, nrow=nrep)
set.seed(100)

for(j in 1:nrep){
 
  # Creating the symbols
  
  classical.data <- vector("list", length = m)
  symbol.data <- vector("list", length = m)
  symbol.data.4 <- vector("list", length = m)
  
  n2 <- n3 <- n4 <- vector(length=m)
  for(i in 1:m){
    tmp <- mvrnorm(ns, mean, Sigma = sigma ) 
    classical.data[[i]] <- tmp
    
    coord <- c(range(tmp[,1]), range(tmp[,2])) # a1, a2, b1, b2
    pts <- c(which.min(tmp[,1]), which.max(tmp[,1]), which.min(tmp[,2]), which.max(tmp[,2]))
    tab <- table(pts)
    extra <- vector("numeric", length=2)
    
    symbol.data.4[[i]] <- list(coord=coord, n=ns, extra=c(4,0) )
        
    if( length(tab) == 2 ){ # Intervals constructed from 2 points
      n2[i] <- 1
      extra[1] = 2
      point <- tmp[as.numeric(names(tab)[1]),]
      if(point[1]==coord[1] && point[2]==coord[3]){extra[2] ="bltr" }
      if(point[1]==coord[1] && point[2]==coord[4]){extra[2] ="tlbr" }
      if(point[1]==coord[2] && point[2]==coord[3]){extra[2] ="tlbr" }
      if(point[1]==coord[2] && point[2]==coord[4]){extra[2] ="bltr" }    
    }
    if( length(tab) == 3){
      n3[i] <- 1
      extra[1] = 3
      point <- tmp[as.numeric(names(tab[which(tab==2)])),]
      if(all(point == coord[c(1,3)])){extra[2] = "bl"}
      if(all(point == coord[c(1,4)])){extra[2] = "tl"}
      if(all(point == coord[c(2,3)])){extra[2] = "br"}
      if(all(point == coord[c(2,4)])){extra[2] = "tr"}
    }
    if(length(tab) == 4){
      n4[i] <- 1
      extra[1] = 4
    }
    symbol.data[[i]] <- list(coord=coord, extra=extra, n=ns)
    
  }  
  
  Classical.data = do.call(rbind, classical.data)
  
  Time <- system.time(
    fit <- fit.biv.int(symbol.data = symbol.data, parastart = c(mean, sd[1], Cor, sd[2]), hessian=FALSE, trace=2)
  )
  
  est[j,] <- c(fit$est , Time[3])
  
  Time.4 <- system.time(
    fit.4 <- fit.biv.int(symbol.data = symbol.data.4, parastart = c(mean, sd[1], Cor, sd[2]), hessian=FALSE, trace=2)
  )
  
  est.4[j,] <- c(fit.4$est, Time.4[3])
  
   
}

save(est, est.4, file= paste("N_",m, "Know&4_Symb_", ns, "PtsSymb_",Cor,"Cor.RData",sep=""))


