### 
### Script to test the Likelihood function for bivariate intervals 
###   constructed from the iterative segmentation method (i.e. L_{is})
###   based on "functions_biv_os2_int.R"
###

library(MASS)
library(mvtnorm)

rm(list=ls())
source("functions_biv_os2_int.R") # Make sure to be in the right directory!

m <- 20 # number of symbols (bivariate intervals)
ns <- 60 # number of points per symbol

mean <- c(2,5)
var <- c(0.5^2, 0.5^2)
Cor <- c(-0.7, 0, 0.7)
Cov <- Cor * prod(sqrt(var))

symbol.os2.data.x <- vector("list", length = m)
symbol.os2.data.y <- vector("list", length = m)
n2 <- n3 <- n4 <- 0 # Count number of intervals constructed with 2, 3 and 4 points 

Orders2.x <- rbind(c(6, 55, 3, 3), c(16, 45, 10, 2), c(20, 41, 7, 14))
Orders2.y <- rbind(c(3, 3, 6, 55), c(10, 2, 16, 45), c(7, 14, 20, 41))

if(ns==300){
  Orders2.x <- Orders2.x * 5
  Orders2.y <- Orders2.y * 5  
}

#
# X-axis construction
# L_{is,x}
#

for( l in 1:nrow(Orders2.x)){
  orders2 <- Orders2.x[l,]
  cat("\n Orders:", orders2, "\n")
  
  for( k in 1:length(Cov)){
    cov=Cov[k]
    cat("\n Covariance:",cov,"\n")
    set.seed(100)
    
    true.param <- c(mean, var[1], cov, var[2])
    
    nrep <- 100 # replicate the simulation nrep times
    est.os2.x <- matrix(ncol=7, nrow=nrep)
    
    pb = txtProgressBar(min = 0, max = nrep, initial = 0, style=3) #creates a prgress bar
    
    for(j in 1:nrep){
      setTxtProgressBar(pb, j)
      
      for(i in 1:m){ 
        tmp <- mvrnorm(ns, mean, Sigma = matrix( c(var[1], cov, cov, var[2]), ncol=2) ) 
                
        coord.A <- sort(tmp[,1])[orders2[1:2]]
        tmp.cond1 <- tmp[(tmp[,1]<coord.A[1]),]
        coord.B <- sort(tmp.cond1[,2])[orders2[3]]
        tmp.cond2 <- tmp[(tmp[,1]>coord.A[2]),]
        coord.C <- sort(tmp.cond2[,2])[orders2[4]]
        coord.os2.x <- c(coord.A, coord.B, coord.C)
        symbol.os2.data.x[[i]] <- list(coord=coord.os2.x, orders=orders2, n=ns)
       
      }
                  
      Time.os2.x <- system.time(
        fit.os2.x <- tryCatch(fit.biv.os2.int.x(symbol.data = symbol.os2.data.x, parastart = true.param, hessian=FALSE, trace=0), error=function(e) list(est=-1))
      )
      
      if(fit.os2.x$est != -1){
        cor.os2.x <- fit.os2.x$est[4]/sqrt(prod(fit.os2.x$est[c(3,5)]))
        est.os2.x[j,] <- c(fit.os2.x$est, cor.os2.x, Time.os2.x[3])
      }
      
    }
    
    save(est.os2.x, file= paste("N_",m, "XSymb_", ns, "PtsSymb_", cov, "Cov_", orders2[1], "_", orders2[2], "_", orders2[3], "_", orders2[4], "_orders.RData", sep=""))
 
  }
  
  
}

#
# Y-axis construction
# L_{is,y}
#

for( l in 1:nrow(Orders2.y)){
  orders2 <- Orders2.y[l,]
  cat("\n Orders:", orders2, "\n")
  
  for( k in 1:length(Cov)){
    cov=Cov[k]
    cat("\n Covariance:",cov,"\n")
    set.seed(100)
    
    true.param <- c(mean, var[1], cov, var[2])
    
    nrep <- 100 # replicate the simulation nrep times
    est.os2.y <- matrix(ncol=7, nrow=nrep)
    
    pb = txtProgressBar(min = 0, max = nrep, initial = 0, style=3) #creates a prgress bar
    
    for(j in 1:nrep){
      setTxtProgressBar(pb, j)
      
      for(i in 1:m){ 
        tmp <- mvrnorm(ns, mean, Sigma = matrix( c(var[1], cov, cov, var[2]), ncol=2) ) 
        
        coord.D <- sort(tmp[,2])[orders2[3:4]]
        tmp.cond3 <- tmp[(tmp[,2]<coord.D[1]),]
        coord.E <- sort(tmp.cond3[,1])[orders2[1]]
        tmp.cond4 <- tmp[(tmp[,2]>coord.D[2]),]
        coord.F <- sort(tmp.cond4[,1])[orders2[2]]
        coord.os2.y <- c(coord.E, coord.F, coord.D)
        symbol.os2.data.y[[i]] <- list(coord=coord.os2.y, orders=orders2, n=ns)
                
      }
         
      Time.os2.y <- system.time(
        fit.os2.y <- tryCatch(fit.biv.os2.int.y(symbol.data = symbol.os2.data.y, parastart = true.param, hessian=FALSE, trace=0), error=function(e) list(est=-1))
      )
      
      if(fit.os2.y$est != -1){
        cor.os2.y <- fit.os2.y$est[4]/sqrt(prod(fit.os2.y$est[c(3,5)]))
        est.os2.y[j,] <- c(fit.os2.y$est, cor.os2.y, Time.os2.y[3])
      }
      
    }
    
    save(est.os2.y, file= paste("N_",m, "YSymb_", ns, "PtsSymb_",cov,"Cov_",orders2[1],"_",orders2[2],"_",orders2[3],"_",orders2[4],"_orders.RData",sep=""))
  }
   
}
