### 
### Script to test the Likelihood function for bivariate intervals 
###   constructed from the sequential nesting method (i.e. L_{sn})
###   based on "functions_biv_os_int.R"
###

library(MASS)
library(mvtnorm)

rm(list=ls())
source("functions_biv_os_int.R") # Make sure to be in the right directory!

m <- 20 # number of symbols (bivariate intervals)
ns <- 60 # number of points per symbol

mean <- c(2,5)
var <- c(0.5^2, 0.5^2)
Cor <- c(-0.7, 0, 0.7)
Cov <- Cor * prod(sqrt(var))

symbol.os.data.x <- symbol.os.data.y <- vector("list", length = m)
n2 <- n3 <- n4 <- 0 # Count number of intervals constructed with 2, 3 and 4 points 

Orders.x <- rbind(c(6, 55, 5, 35), c(16, 45, 6, 24), c(20, 41, 5, 16))
Orders.y <- rbind(c(5, 35, 6, 55), c(6, 24, 16, 45), c(5, 16, 20, 41))

if(ns==300){
  Orders.x <- Orders.x * 5
  Orders.y <- Orders.y * 5 
}

#
# X-axis construction
# L_{sn,x}
#

for( l in 1:nrow(Orders2.x)){
  
  orders <- Orders.x[l,]
  cat("\n Orders:", orders, "\n")
  
  for( k in 1:length(Cov)){
    cov=Cov[k]
    cat("\n Covariance:",cov,"\n")
    set.seed(100)
    
    true.param <- c(mean, var[1], cov, var[2])
    
    nrep <- 100 # replicate the simulation nrep times
    est.os.x <- matrix(ncol=7, nrow=nrep)
    
    pb = txtProgressBar(min = 0, max = nrep, initial = 0, style=3) #creates a prgress bar
    
    for(j in 1:nrep){
      setTxtProgressBar(pb, j)
      
      for(i in 1:m){
        tmp <- mvrnorm(ns, mean, Sigma = matrix( c(var[1], cov, cov, var[2]), ncol=2) ) 
        
        coord.1 <- sort(tmp[,1])[orders[1:2]]
        tmp.cond <- tmp[(tmp[,1]>coord.1[1]) & (tmp[,1]<coord.1[2]),]
        coord.2 <- sort(tmp.cond[,2])[orders[3:4]]
        coord.os <- c(coord.1, coord.2)
        symbol.os.data.x[[i]] <- list(coord=coord.os, orders=orders, n=ns)
        
      }

      Time.os.x <- system.time(
        fit.os.x <- tryCatch(fit.biv.os.int.x(symbol.data = symbol.os.data.x, parastart = true.param, hessian=FALSE, trace=0), error=function(e) list(est=-1))
      )

      if(fit.os.x$est != -1){
        cor.os.x <- fit.os.x$est[4]/sqrt(prod(fit.os.x$est[c(3,5)]))
        est.os.x[j,] <- c(fit.os.x$est, cor.os.x, Time.os.x[3])
      }
      
    }
    
    save(est.os.x, file= paste("N_",m, "CentreXSymb_", ns, "PtsSymb_",cov,"Cov_",orders[1],"_",orders[2],"_",orders[3],"_",orders[4],"_orders.RData",sep=""))
  }
  
}

#
# Y-axis construction
# L_{sn,y}
#

for( l in 1:nrow(Orders2.y)){
  orders <- Orders.y[l,]
  cat("\n Orders:", orders, "\n")
  
  for( k in 1:length(Cov)){
    cov=Cov[k]
    cat("\n Covariance:",cov,"\n")
    set.seed(100)
    
    true.param <- c(mean, var[1], cov, var[2])
    
    nrep <- 100 # replicate the simulation nrep times
    est.os.y <- matrix(ncol=7, nrow=nrep)
    
    pb = txtProgressBar(min = 0, max = nrep, initial = 0, style=3) #creates a prgress bar
    
    for(j in 1:nrep){
      setTxtProgressBar(pb, j)
      
      for(i in 1:m){
        tmp <- mvrnorm(ns, mean, Sigma = matrix( c(var[1], cov, cov, var[2]), ncol=2) ) 
        
        coord.2 <- sort(tmp[,2])[orders[3:4]]
        tmp.cond <- tmp[(tmp[,2]>coord.2[1]) & (tmp[,2]<coord.2[2]),]
        coord.1 <- sort(tmp.cond[,1])[orders[1:2]]
        coord.os <- c(coord.1, coord.2)
        symbol.os.data.y[[i]] <- list(coord=coord.os, orders=orders, n=ns)
                
      }
      
      Time.os.y <- system.time(
        fit.os.y <- tryCatch(fit.biv.os.int.y(symbol.data = symbol.os.data.y, parastart = true.param, hessian=FALSE, trace=0), error=function(e) list(est=-1))
      )

      if(fit.os.y$est != -1){
        cor.os.y <- fit.os.y$est[4]/sqrt(prod(fit.os.y$est[c(3,5)]))
        est.os.y[j,] <- c(fit.os.y$est, cor.os.y, Time.os.y[3])
      }
      
    }
    
    save(est.os.y, file= paste("N_",m, "CentreYSymb_", ns, "PtsSymb_",cov,"Cov_",orders[1],"_",orders[2],"_",orders[3],"_",orders[4],"_orders.RData",sep=""))   
  }
   
}

